@@ -51,7 +51,7 @@ let string_of_kind = function
 
 let rec longident_of_path = function
     Pident id -> Lident (Ident.name id)
-  | Pdot (path, s, _) -> Ldot (longident_of_path path, s)
+  | Pdot (path, s) -> Ldot (longident_of_path path, s)
   | Papply (p1, p2) -> Lapply (longident_of_path p1, longident_of_path p2)
 
 let rec remove_prefix lid ~prefix =
@@ -128,7 +128,7 @@ let rec equal ~prefix t1 t2 =
       begin fun l1 ->
         List.for_all2 l1 l2 ~f:
         begin fun (p1,t1) (p2,t2) ->
-          (p1 = "" || p1 = p2) && equal t1 t2 ~prefix
+          (p1 = Nolabel || p1 = p2) && equal t1 t2 ~prefix
         end
       end
   | Ttuple l1, Ttuple l2 ->
@@ -140,8 +140,7 @@ let rec equal ~prefix t1 t2 =
       && List.for_all2 l1 l2 ~f:(equal ~prefix)
   | _ -> false
 
-let is_opt s = s <> "" && s.[0] = '?'
-let get_options = List.filter ~f:is_opt
+let get_options = List.filter ~f:Btype.is_optional
 
 let rec included ~prefix t1 t2 =
   match (repr t1).desc, (repr t2).desc with
@@ -170,14 +169,14 @@ let rec included ~prefix t1 t2 =
       let l2 = if arr len1 ~card:len2 < 100 then l2 else
           let ll1 = get_options (fst (List.split l1)) in
           List.filter l2
-          ~f:(fun (l,_) -> not (is_opt l) || List.mem l ll1)
+          ~f:(fun (l,_) -> not (is_optional l) || List.mem l ll1)
       in
       len1 <= len2 &&
       List.exists (List2.flat_map ~f:permutations (choose len1 ~card:l2)) ~f:
       begin fun l2 ->
         List.for_all2 l1 l2 ~f:
         begin fun (p1,t1) (p2,t2) ->
-          (p1 = "" || p1 = p2) && included t1 t2 ~prefix
+          (p1 = Nolabel || p1 = p2) && included t1 t2 ~prefix
         end
       end
   | Ttuple l1, Ttuple l2 ->
@@ -202,11 +201,12 @@ let mklid = function
 let mkpath = function
     [] -> raise (Invalid_argument "Searchid.mklid")
   | x :: l ->
-      List.fold_left l ~init:(Pident (Ident.create x))
-      ~f:(fun acc x -> Pdot (acc, x, 0))
+      List.fold_left l ~init:(Pident (Ident.create_local x))
+      ~f:(fun acc x -> Pdot (acc, x))
 
 let get_fields ~prefix ~sign self =
-  let env = open_signature Fresh (mkpath prefix) sign !start_env in
+  (*let env = open_signature Fresh (mkpath prefix) sign !start_env in*)
+  let env = add_signature sign !start_env in
   match (expand_head env self).desc with
     Tobject (ty_obj, _) ->
       let l,_ = flatten_fields ty_obj in l
@@ -217,11 +217,15 @@ let rec search_type_in_signature t ~sign ~prefix ~mode =
         `Included -> included t ~prefix
       | `Exact -> equal t ~prefix
   and lid_of_id id = mklid (prefix @ [Ident.name id]) in
+  let constructor_matches = function
+      Types.Cstr_tuple l -> List.exists l ~f:matches
+    | Cstr_record l -> List.exists l ~f:(fun d -> matches d.Types.ld_type)
+  in
   List2.flat_map sign ~f:
   begin fun item -> match item with
-        Sig_value (id, vd) ->
+        Sig_value (id, vd, _) ->
           if matches vd.val_type then [lid_of_id id, Pvalue] else []
-      | Sig_type (id, td, _) ->
+      | Sig_type (id, td, _, _) ->
           if
           matches (newconstr (Pident id) td.type_params) ||
           begin match td.type_manifest with
@@ -233,31 +237,31 @@ let rec search_type_in_signature t ~sign ~prefix ~mode =
 	  | Type_open -> false
           | Type_variant l ->
             List.exists l ~f:
-            begin fun {Types.cd_args=l; cd_res=r} ->
-              List.exists l ~f:matches ||
+            begin fun {Types.cd_args=args; cd_res=r} ->
+              constructor_matches args  ||
               match r with None -> false | Some x -> matches x
             end
           | Type_record(l, rep) ->
             List.exists l ~f:(fun {Types.ld_type=t} -> matches t)
           end
           then [lid_of_id id, Ptype] else []
-      | Sig_typext (id, l, _) ->
-          if List.exists l.ext_args ~f:matches
+      | Sig_typext (id, l, _, _) ->
+          if constructor_matches l.ext_args
           then [lid_of_id id, Pconstructor]
           else []
-      | Sig_module (id, {md_type=Mty_signature sign}, _) ->
+      | Sig_module (id, _, {md_type=Mty_signature sign}, _, _) ->
           search_type_in_signature t ~sign ~mode
             ~prefix:(prefix @ [Ident.name id])
       | Sig_module _ -> []
       | Sig_modtype _ -> []
-      | Sig_class (id, cl, _) ->
+      | Sig_class (id, cl, _, _) ->
           let self = self_type cl.cty_type in
           if matches self
           || (match cl.cty_new with None -> false | Some ty -> matches ty)
           (* || List.exists (get_fields ~prefix ~sign self)
               ~f:(fun (_,_,ty_field) -> matches ty_field) *)
           then [lid_of_id id, Pclass] else []
-      | Sig_class_type (id, cl, _) ->
+      | Sig_class_type (id, cl, _, _) ->
           let self = self_type cl.clty_type in
           if matches self
           (* || List.exists (get_fields ~prefix ~sign self)
@@ -270,7 +274,7 @@ let search_all_types t ~mode =
       `Exact, _ -> [t]
     | `Included, Tarrow _ -> [t]
     | `Included, _ ->
-      [t; newty(Tarrow("",t,newvar(),Cok)); newty(Tarrow("",newvar(),t,Cok))]
+      [t; newty(Tarrow(Nolabel,t,newvar(),Cok)); newty(Tarrow(Nolabel,newvar(),t,Cok))]
   in List2.flat_map !module_list ~f:
     begin fun modname ->
     let mlid = Lident modname in
@@ -304,7 +308,7 @@ let search_string_type text ~mode =
           let end_c = l.loc_end.Lexing.pos_cnum in
           raise (Error (start_c - 8, end_c - 8))
     in match sign with
-        [ Sig_value (_, vd) ] ->
+        [ Sig_value (_, vd, _) ] ->
           search_all_types vd.val_type ~mode
       | _ -> []
   with
@@ -362,17 +366,17 @@ let search_pattern_symbol text =
     with {md_type=Mty_signature sign} ->
         List2.flat_map sign ~f:
           begin function
-            Sig_value (i, _) when check i -> [i, Pvalue]
-          | Sig_type (i, _, _) when check i -> [i, Ptype]
-          | Sig_typext (i, _, _) when check i -> [i, Pconstructor]
-          | Sig_module (i, _, _) when check i -> [i, Pmodule]
-          | Sig_modtype (i, _) when check i -> [i, Pmodtype]
-          | Sig_class (i, cl, _) when check i
+            Sig_value (i, _, _) when check i -> [i, Pvalue]
+          | Sig_type (i, _, _, _) when check i -> [i, Ptype]
+          | Sig_typext (i, _, _, _) when check i -> [i, Pconstructor]
+          | Sig_module (i, _, _, _, _) when check i -> [i, Pmodule]
+          | Sig_modtype (i, _, _) when check i -> [i, Pmodtype]
+          | Sig_class (i, cl, _, _) when check i
             || List.exists
                 (get_fields ~prefix:[modname] ~sign (self_type cl.cty_type))
                 ~f:(fun (name,_,_) -> check_match ~pattern (explode name))
             -> [i, Pclass]
-          | Sig_class_type (i, cl, _) when check i
+          | Sig_class_type (i, cl, _, _) when check i
             || List.exists
                 (get_fields ~prefix:[modname] ~sign (self_type cl.clty_type))
                 ~f:(fun (name,_,_) -> check_match ~pattern (explode name))
@@ -434,6 +438,7 @@ let rec bound_variables pat =
   | Ppat_lazy pat -> bound_variables pat
   | Ppat_extension _ -> []
   | Ppat_exception pat -> bound_variables pat
+  | Ppat_open (_, pat) -> bound_variables pat
 
 let search_structure str ~name ~kind ~prefix =
   let loc = ref 0 in
@@ -465,7 +470,7 @@ let search_structure str ~name ~kind ~prefix =
             end;
           false
       | Pstr_primitive vd when kind = Pvalue -> name = vd.pval_name.txt
-      | Pstr_type l when kind = Ptype ->
+      | Pstr_type (_, l) when kind = Ptype ->
           List.iter l ~f:
             begin fun td ->
               if td.ptype_name.txt = name
@@ -479,7 +484,8 @@ let search_structure str ~name ~kind ~prefix =
 	      then loc := td.pext_loc.loc_start.Lexing.pos_cnum
             end;
           false
-      | Pstr_exception pcd when kind = Pconstructor -> name = pcd.pext_name.txt
+      | Pstr_exception pcd when kind = Pconstructor ->
+          name = pcd.ptyexn_constructor.pext_name.txt
       | Pstr_module x when kind = Pmodule -> name = x.pmb_name.txt
       | Pstr_modtype x when kind = Pmodtype -> name = x.pmtd_name.txt
       | Pstr_class l when kind = Pclass || kind = Ptype || kind = Pcltype ->
@@ -526,7 +532,7 @@ let search_signature sign ~name ~kind ~prefix =
     begin fun item ->
       if match item.psig_desc with
         Psig_value vd when kind = Pvalue -> name = vd.pval_name.txt
-      | Psig_type l when kind = Ptype ->
+      | Psig_type (_, l) when kind = Ptype ->
           List.iter l ~f:
             begin fun td ->
               if td.ptype_name.txt = name
@@ -540,7 +546,8 @@ let search_signature sign ~name ~kind ~prefix =
 	      then loc := td.pext_loc.loc_start.Lexing.pos_cnum
             end;
           false
-      | Psig_exception pcd when kind = Pconstructor -> name = pcd.pext_name.txt
+      | Psig_exception pcd when kind = Pconstructor ->
+          name = pcd.ptyexn_constructor.pext_name.txt
       | Psig_module pmd when kind = Pmodule -> name = pmd.pmd_name.txt
       | Psig_modtype pmtd when kind = Pmodtype -> name = pmtd.pmtd_name.txt
       | Psig_class l when kind = Pclass || kind = Ptype || kind = Pcltype ->
