@@ -167,7 +167,7 @@ let option_hack oc =
 let realname name =
   (* module name fix for camltk *)
   let name = caml_name name in
-  if !Flags.camltk then "c" ^ String.capitalize name
+  if !Flags.camltk then "c" ^ String.capitalize_ascii name
   else name
 ;;
 
@@ -291,8 +291,8 @@ let compile () =
     Hashtbl.iter (fun name _ ->
       let cname = realname name in
       output_string oc (Printf.sprintf "module %s = %s;;\n"
-                          (String.capitalize (caml_name name))
-                          (String.capitalize cname))) module_table;
+                          (String.capitalize_ascii (caml_name name))
+                          (String.capitalize_ascii cname))) module_table;
     close_out oc
   end else begin
     let oc = open_out_bin (destfile "labltk.ml") in
@@ -312,8 +312,8 @@ module Timer = Timer;;\n\
     Hashtbl.iter (fun name _ ->
       let cname = realname name in
       output_string oc (Printf.sprintf "module %s = %s;;\n"
-                          (String.capitalize (caml_name name))
-                          (String.capitalize cname))) module_table;
+                          (String.capitalize_ascii (caml_name name))
+                          (String.capitalize_ascii cname))) module_table;
     (* widget typer *)
     output_string oc "\n(** Widget typers *)\n\nopen Widget\n\n";
     Hashtbl.iter (fun name def ->
